// Stencil
import { h, Component, Prop } from '@stencil/core';
import { MarkdownTool } from '../../../../tools/markdown-tool';
export class TourDetails {
    constructor() {
        // Props
        this.detailsTitle = '';
        this.price = '';
        this.duration = '';
        this.bestMoment = '';
        this.address = '';
        this.exitPrefix = '';
        this.transportInfo = [];
        this.city = 'paris';
    }
    ////////////////////////////////////////////////////////////////
    // Render
    ////////////////////////////////////////////////////////////////
    render() {
        return (h("section", { id: 'details' },
            h("div", { class: 'statistics-container' },
                h("h4", { class: 'composition-section-title' }, this.detailsTitle),
                h("ion-list", { class: 'statistics-list' },
                    h("ion-item", null,
                        h("div", { class: 'details-icon-container' },
                            h("ion-icon", { name: 'hourglass', color: 'dark' })),
                        h("div", null,
                            h("p", null, this.duration))),
                    typeof this.price !== 'undefined' ?
                        h("ion-item", null,
                            h("div", { class: 'details-icon-container' },
                                h("ion-icon", { name: 'pricetags', color: 'dark' })),
                            h("div", null,
                                h("p", null, this.price)))
                        : '',
                    h("ion-item", null,
                        h("div", { class: 'details-icon-container' },
                            h("ion-icon", { name: 'time', color: 'dark' })),
                        h("div", { innerHTML: MarkdownTool.render(this.bestMoment), class: 'details-container' })),
                    h("ion-item", null,
                        h("div", { class: 'details-icon-container' },
                            h("ion-icon", { name: 'pin', color: 'dark' })),
                        h("div", { class: 'details-container' },
                            h("p", null, this.address))),
                    typeof this.transportInfo !== 'undefined' ?
                        h("ion-item", null,
                            h("div", { class: 'details-icon-container' },
                                h("ion-icon", { name: 'bus', color: 'dark' })),
                            h("div", { class: 'details-container' },
                                h("metro-indication", { transportInfo: this.transportInfo, city: this.city, exitPrefix: this.exitPrefix, colorMode: 'dark' })))
                        : ''))));
    }
    static get is() { return "tour-details"; }
    static get originalStyleUrls() { return {
        "$": ["tour-details.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["tour-details.css"]
    }; }
    static get properties() { return {
        "detailsTitle": {
            "type": "string",
            "mutable": true,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "details-title",
            "reflect": false,
            "defaultValue": "''"
        },
        "price": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "price",
            "reflect": false,
            "defaultValue": "''"
        },
        "duration": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "duration",
            "reflect": false,
            "defaultValue": "''"
        },
        "bestMoment": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "best-moment",
            "reflect": false,
            "defaultValue": "''"
        },
        "address": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "address",
            "reflect": false,
            "defaultValue": "''"
        },
        "exitPrefix": {
            "type": "string",
            "mutable": true,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "exit-prefix",
            "reflect": false,
            "defaultValue": "''"
        },
        "transportInfo": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "TransportIndication[]",
                "resolved": "TransportIndication[]",
                "references": {
                    "TransportIndication": {
                        "location": "import",
                        "path": "../../../../models/transport/transport-indication"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "defaultValue": "[]"
        },
        "city": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "city",
            "reflect": false,
            "defaultValue": "'paris'"
        }
    }; }
}
